#!/usr/bin/env python

# Feb. 2016, wwortel
# version: 0.06

import sys, os, binascii
from sys import platform as _platform

####################################
# EDIT THE NAME OF THE INPUT FILE
# 'dvb_s_mw_s1' for sftp to th.dtv
# 'mw_s1' for an exported file
# to be present where this script is
####################################
s1_name = 'dvb_s_mw_s1'

####################################
# adapt codeset to best represent
# special characters.
# https://docs.python.org/2/library/codecs.html#standard-encodings
####################################
codeset = "cp860"
####################################

if _platform == "linux" or _platform == "linux2" or _platform == "darwin":
  # linux & MAC OS X
  slash = '/'
elif _platform == "win32":
  # Windows
  slash = "\\" # a single \ would make the " literal
dhere = os.path.realpath('.')
s1_name = os.path.join(dhere, s1_name)

endpatt = bytearray((int(0x00), int(0x00),int(0x3f), int(0xff)))

def error(fmt, *msg):
  sys.stderr.write(" [ERROR] " + (fmt % msg) + '\n')
  sys.exit(1)

def pmsg(fmt, *msg):
  print(fmt % msg)

if not os.path.isfile(s1_name):
  error('input binary file "%s" not found', s1_name)

with open(s1_name, 'rb') as binfile:
  bindata = bytearray(binfile.read())

binl = len(bindata)

# check file length integrity
givl = bindata[ 0 : 4 ] # [n:m] first n : up to but not including m
dcmll = int(binascii.hexlify(givl), 16) + 4

if not binl == dcmll:
  error('length of input binary file "%s" \n differs from length given in that file!', s1_name)

def find_end(strt):
  for sidx in range(strt,strt+300,1): # 300 to contain run-aways
    if bindata[ sidx : sidx + 4 ] == endpatt:
      break
  return sidx + 4 # index of next record start

recd_nmbr = int(binascii.hexlify(bindata[12:16]), 16)

# initial indices of record and bindata
recd_idx = 1
bind_idx = 16
while recd_idx <= recd_nmbr: 
  nxt_idx = find_end(bind_idx)
  binrcd = bindata[bind_idx:nxt_idx]
  rcdlen = nxt_idx - bind_idx
# create record file name
  rcdnamel = int(binrcd[1])
# tidy up the service names to make proper file names
  rcdname = binrcd[5:rcdnamel + 5]
  for idx in range(0,rcdnamel):
#   rcdname[idx] &= int(0x7f)
    if rcdname[idx] < 0x21:  # non printable codes or spaces
      rcdname[idx] = 0x5f    # an underscore instead
    if ( rcdname[idx] == 0x7f or rcdname[idx] == 0x3c or rcdname[idx] == 0x3a or rcdname[idx] == 0x22 or 
         rcdname[idx] == 0x2f or rcdname[idx] == 0x5c or rcdname[idx] == 0x7c or rcdname[idx] == 0x3f or
         rcdname[idx] == 0x2a or rcdname[idx] == 0x3e
       ):  # control chars < > : " / \ | ? * space are not allowed in file names; replace by %
      rcdname[idx] = 0x25
# start the filename with R | TV | ? to indicate the radio/TV/unknown service type
  stype = "U" # unknown
  if binrcd[rcdlen - 17] == 0x00: # this byte in fixed distance 17 back from next record has TV/Radio
    stype = "TV"
  elif binrcd[rcdlen - 17] == 0x01:
    stype = "R"
  nid = binrcd[ rcdlen - 26 : rcdlen - 24 ] # also fixed distance back from end
  nidint = int(binascii.hexlify(nid), 16) # make the two bytes into an integer
  ppr = binrcd[ rcdlen - 10 : rcdlen - 8 ] # preference setting
  pprint = int(binascii.hexlify(ppr), 16)
# add the network number and preference setting to the end of the file name
  asciiname = stype + "~" + str(recd_idx) + "~" + rcdname.decode(codeset) + "~E0~" + "N" + str(nidint) + "~" + "P" + str(pprint)
  dirpath = dhere + slash + 'services_all'
  filepath = os.path.join(dirpath, asciiname )
  with open(filepath, 'wb') as rcdfile:
    rcdfile.write(binrcd)
  recd_idx += 1
  bind_idx = nxt_idx

pmsg('Total number of service files written: %d.', recd_idx - 1)


